
#include <MIDI.h>
//Tested with version 4.2 of the Arduino MIDI library

//PracticalUsage.com 2015.
//This is Open Source.
// MP-4 Midi-Processor for the Akai MPK261
// 001 Use NoteOn and CC to control Pads color

//Use MySettings to change the default settings for the Midi Library
struct MySettings : 
public midi::DefaultSettings
{
  //    static const bool UseRunningStatus = false; // Messes with my old equipment!
  //    static const bool 	UseRunningStatus = true;
  //    static const bool 	HandleNullVelocityNoteOnAsNoteOff = true;
  //    static const bool 	Use1ByteParsing = true;
  //    static const long 	BaudRate = 31250;
  //    static const unsigned 	SysExMaxSize = 128;
  static const unsigned SysExMaxSize = 128;
};

// This is for an Arduino Mega with several serial ports. use Serial for Arduino
MIDI_CREATE_CUSTOM_INSTANCE(HardwareSerial, Serial1, midiA, MySettings);

//Variables and constants
int LED = 13;          // LED pin on Arduino
byte CC; //storage for Control Change
boolean blinking = false;
long blinkMillis;
boolean keepBlinking;
int noteNumber; //note number within a scal (1-12)
int scaleNumber; //scale number (1-8)
byte drumPads1 [] = {
  1,4,12,2,2,5,7,9,3,10,11,5,8,1,3,12}; // Just an expample
byte warningPads [] = { 
  1,0,0,0,0,1,0,0,0,0,0,0,0,0,0,2}; // Just an example
boolean showTime = false; //Easter Egg!
int MPK2PadsSysExLength = 12;
byte MPK2PadsSysEx [] = {
  0xF0,0x47,0x00,0x25,0x31,0x00,0x04,0x01,0x0B,0x00,0x00,0xF7};
long randomNumber = 0;


void setup()
{
  pinMode(LED, OUTPUT);
  midiA.begin(MIDI_CHANNEL_OMNI);          // Launch MIDI and listen to all channels
  midiA.turnThruOff(); //Don't echo MIDI messages on the output

}

void loop()
{


  if (midiA.read())                // If we have received a midi message
  {

    if (midiA.getType() == midi::ControlChange) { //is the midi message a CC?
      showTime = false;
      CC = midiA.getData1();
      switch (CC) {
      case 32:
        loadDarkPads();
        break;  
      case 33:
        blinking = false;
        loadDrumPads();
        break;
      case 34:
        blinking = true;
        keepBlinking = true;
        blinkMillis = millis();
        loadWarning();
        break;
      }
    }
    if (midiA.getType() == midi::NoteOn) { //is the midi message a NoteOn?
      blinking = false;
      noteNumber = midiA.getData1();
      scaleNumber = noteNumber / 12;
      noteNumber = noteNumber%12;
      MPK2PadsSysEx [9] = noteNumber; //which pad?
      MPK2PadsSysEx [10] = scaleNumber + 1; //which color
      midiA.sendSysEx( MPK2PadsSysExLength, MPK2PadsSysEx, true); //don't send F0 & F7
      if (midiA.getData1() == 96) {
        showTime = true;// Random Pad lights up!
      }
    }
    if (midiA.getType() == midi::NoteOff) { //is the midi message a Noteoff?
      blinking = false;
      noteNumber = midiA.getData1();
      scaleNumber = noteNumber / 12;
      noteNumber = noteNumber%12;
      MPK2PadsSysEx [9] = noteNumber; //which pad?
      MPK2PadsSysEx [10] = 0; //which color = Dark
      midiA.sendSysEx( MPK2PadsSysExLength, MPK2PadsSysEx, true); //don't send F0 & F7
      if (midiA.getData1() == 96) {
        showTime = false;
      }
    }
  }

  if (showTime) {
    MPK2PadsSysEx [9] = int(random(12));
    MPK2PadsSysEx [10] = int(random(1,16));
    midiA.sendSysEx( MPK2PadsSysExLength, MPK2PadsSysEx, true); //don't send F0 & F7
    delay(500);
    MPK2PadsSysEx [10] = 0;
    midiA.sendSysEx( MPK2PadsSysExLength, MPK2PadsSysEx, true);

  }

  if (blinking) {
    if (keepBlinking) {
      if (millis() - blinkMillis > 500) {
        loadDarkPads() ;
        blinkMillis = millis();
        keepBlinking = false;
      }
    } 
    else {
      if (millis() - blinkMillis > 500) {
        loadWarning();
        blinkMillis = millis();
        keepBlinking = true;
      }
    }

  }

}

void loadDarkPads() {
  byte a;
  // first row of pads is 0X0A7C to 0X0A7F
  MPK2PadsSysEx [8] = 0x0A; //pad address is 2 bytes. First one is 0x0A or 0x0B
  for ( a = 0x7C ; a <= 0X7F ; a++) { //second byte varies depending on first byte
    MPK2PadsSysEx [9] = a; //Which pad
    MPK2PadsSysEx [10] = 0; //What color (here Dark)
    midiA.sendSysEx( MPK2PadsSysExLength, MPK2PadsSysEx, true);
  }
  // second to fourth row of pad is 0X0B00 to 0X0B0C
  MPK2PadsSysEx [8] = 0x0B;
  for ( a = 0x00 ; a <= 0x0C ; a++) {
    MPK2PadsSysEx [9] = a;
    MPK2PadsSysEx [10] = 0;
    midiA.sendSysEx( MPK2PadsSysExLength, MPK2PadsSysEx, true);
  }
}

void loadDrumPads() {
  byte a;
  MPK2PadsSysEx [8] = 0x0A;
  for ( a = 0x7C ; a <= 0X7F ; a++) {
    MPK2PadsSysEx [9] = a;
    MPK2PadsSysEx [10] = drumPads1 [a - 0x7C]; //predefined color array for each pad. First row
    midiA.sendSysEx( MPK2PadsSysExLength, MPK2PadsSysEx, true);
  }
  MPK2PadsSysEx [8] = 0x0B;
  for ( a = 0x00 ; a <= 0x0C ; a++) {
    MPK2PadsSysEx [9] = a;
    MPK2PadsSysEx [10] = drumPads1 [a + 4]; //second to 4th row
    midiA.sendSysEx( MPK2PadsSysExLength, MPK2PadsSysEx, true);
  }
}

void loadWarning() {
  //example of direct pad adressing for use in blinking one or more pads 
  byte a;
  MPK2PadsSysEx [8] = 0x0A;
  for ( a = 0x7C ; a <= 0X7F ; a++) {
    MPK2PadsSysEx [9] = a;
    MPK2PadsSysEx [10] = warningPads [a - 0x7C]; //predefined color array for each pad. First row
    midiA.sendSysEx( MPK2PadsSysExLength, MPK2PadsSysEx, true);
  }
  MPK2PadsSysEx [8] = 0x0B;
  for ( a = 0x00 ; a <= 0x0C ; a++) {
    MPK2PadsSysEx [9] = a;
    MPK2PadsSysEx [10] = warningPads [a + 4]; //second to 4th row
    midiA.sendSysEx( MPK2PadsSysExLength, MPK2PadsSysEx, true);
  }
}





